/*
 * GstartLed.c
 *
 * Created: 12.05.2017 10:33:37
 *  Author: dsosnowski
 */ 




#define F_CPU 1000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "wdog.h"

//------------------------------------------------------------------
#define OUT_PIN				0		//H
#define PWM_TIM				TIMER0A
#define LED_PIN				1		//H
#define REG_PIN				4		//A
#define REG_CH				2		//ADC

#define PWM_RESOLUTION		255
#define PWM_START_INDEX		0
#define PWM_STOP_INDEX		63

#define LED_DUTY			10
#define LED_PERIOD			150

void HardInit(void);
void AnalogInit(void);
void PwmInit(void);
void PwmEnable(void);
void PwmDisable(void);


//------------------------------------------------------------------
struct {
	uint8_t duty;
	uint8_t index;
	uint8_t delay;
	uint8_t reg;
} pwm;

struct {
	uint16_t duty;
	uint8_t enable;
} led;


const char pwrarray[] = {
	/*0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	16, 18, 20, 23, 25, 28, 30, 33, 36, 39, 42, 46, 49, 53, 56, 60,
	64, 68, 72, 77, 81, 86, 90, 95,100,105,110,116,121,127,132,138,
	144,150,156,163,169,176,182,189,196,203,210,218,225,233,242,255*/
	
	0,	1,	2,	3,	4,	5,	6,	7,	8,	9,	10,	11,	12,	13,	14,	15,
	16,	18,	20,	22,	24,	27,	30,	33,	36,	39,	42,	45,	48,	52,	56,	60,
	64,	68,	72,	76,	80,	85,	90,	95,	100,105,110,115,120,126,132,138,
	144,150,156,162,168,174,181,188,195,202,209,217,225,233,242,255
};



//------------------------------------------------------------------
int main(void)
{	
	HardInit();
	AnalogInit();
	PwmInit();
	
	pwm.duty = 0;
	pwm.delay = 0xFF;
	pwm.index = 0;
	
    while(1)
    {
		_delay_ms(1);
		
		pwm.reg = (ADCH >> 0);
		if (pwm.reg < 3) pwm.reg = 3;
		if (pwm.delay < pwm.reg){
			pwm.delay++;
		} else {
			pwm.delay = 0;
			
			if (pwm.index < PWM_STOP_INDEX) {
				pwm.index++;				
				pwm.duty = pwrarray[pwm.index];
				OCR0A = (0xFF - pwm.duty);
				PORTB &= ~(1<<OUT_PIN);					
				PwmEnable();
				led.enable = 1;
			} else {
				PORTB |= (1<<OUT_PIN);
				PwmDisable();
				led.enable = 0;				
			}
		}
		
		if (led.enable > 0){
			if (led.duty < LED_DUTY){ 
				PORTB |= (1<<LED_PIN);
			} else {
				PORTB &= ~(1<<LED_PIN);			
			}
			led.duty++;
			
			if (led.duty >= LED_PERIOD){
				led.duty = 0;
			}
		} else {
				PORTB |= (1<<LED_PIN);			
		}
		
		WdtReset();
    }
}

//------------------------------------------------------------------
void HardInit(void){
	PORTB = 0;
	DDRB = (1<<LED_PIN | 1<<OUT_PIN);
	ACSR = (1<<ACD);
	PRR = (1<<PRTIM1 | 1<<PRUSI);
	WdtSet(7);
}

//----------------------------------
void AnalogInit(void){
	DIDR0 = (1<<ADC2D);
	ADCSRB = 0;
	ADCSRA = (1<<ADEN | 1<<ADATE | 1<<ADPS2 | 1<<ADPS1);
	ADMUX = (1<<ADLAR | REG_CH);
	ADCSRA |= (1<<ADSC);
}

//----------------------------------
void PwmInit(void){	
	TCCR0A = (1<<WGM01 | 1<<WGM00);		//fast pwm
	TCCR0B = (1<<CS01);
}

//----------------------------------
void PwmEnable(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00 | 1<<COM0A1 | 1<<COM0A0);
}

//----------------------------------
void PwmDisable(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00);
}
